/*
  Copyright 2005  Artima Software, Inc.
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
  express or implied. See the License for the specific language governing
  permissions and limitations under the License.
*/

package com.artima.examples.jackrabbit;

import org.apache.jackrabbit.core.jndi.RegistryHelper;
import org.apache.jackrabbit.core.value.StringValue;
import org.apache.jackrabbit.core.value.DateValue;

import javax.jcr.*;
import javax.jcr.version.Version;
import javax.jcr.version.VersionHistory;
import javax.jcr.version.VersionIterator;
import javax.jcr.query.QueryManager;
import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Hashtable;
import java.util.Calendar;
import java.text.SimpleDateFormat;


/**
 * A simple demo of the Jackrabbit repository that allows one to add and list nodes
 * in a repository.
 */
public class RepoTest {

    //The repostory config file needed to initialize the Jackrabbit
    //repository. If the repository is obtained via a naming service,
    //for instance, then this information is not required for a client program.
    private static final String CONFIG_FILE = "repository.xml";

    //The directory for the repository files.
    private static final String REPO_HOME_DIR = "repotest";

    //The repository's JNDI name
    private static final String REPO_NAME = "repo";

    //User ID and password to log into the repository
    private static final String USERID = "userid";
    private static final char[] PASSWORD = "".toCharArray();

    private static final SimpleDateFormat df = new SimpleDateFormat("MM/dd/yyyy  HH:mm");


    private static Context ctx = null;

    /**
     * The following command-line switches are possible:
     * <p/>
     * <code>-add BLOGGER BLOGTITLE BLOGTEXT</code> Add a blog entry for the
     * given author, with the given title and text.
     * <p/>
     * <code>-update BLOGGER BLOGTITLE NEWTEXT</code> Update the specified blog.
     * <p/>
     * <code>-list BLOGGER</code> List entries written by the specified blogger.
     * <p/>
     * <code>-versions BLOGGER BLOGTITLE</code> List all versions of the blog entry with
     * the specified blogger and title.
     */
    public static void main(String[] args) {

        //So we don't have to type this on the command line every time
        System.setProperty("java.security.auth.login.config", "jaas.config");

        if (args.length < 2) {
            throw new IllegalArgumentException(usage());
        }

        String command = args[0];
        if (command.equals("-add")) {
            if (args.length < 4) {
                throw new IllegalArgumentException(usage());
            }
            addBlogEntry(args[1], args[2], args[3]);

        } else if (command.equals("-list")) {
            printEntriesByBlogAuthor(args[1]);

        } else if (command.equals("-versions")) {
            if (args.length < 3) {
                throw new IllegalArgumentException(usage());
            }
            printVersions(args[1], args[2]);

        } else if (command.equals("-update")) {
            if (args.length < 4) {
                throw new IllegalArgumentException(usage());
            }
            try {
                updateBlogEntry(args[1], args[2], args[3]);
            } catch (ItemNotFoundException e) {
                System.out.println("Could not find a blog item with author " + args[1] + " and title " + args[2]);
            }
        } else {
            throw new IllegalArgumentException(usage());
        }
    }

    private static void updateBlogEntry(String blogAuthor, String blogTitle, String blogtext) throws ItemNotFoundException {
        if (blogAuthor == null || blogTitle == null || blogtext == null) {
            throw new NullPointerException();
        }

        Session session = null;
        try {
            Repository r = getRepository();

            Credentials credentials = new SimpleCredentials(USERID, PASSWORD);
            session = r.login(credentials);

            //Find the blog with the specified author and text. If no such blog exists, indicate that to the user
            Workspace workspace = session.getWorkspace();
            NodeIterator iterator = findBlog(workspace, blogAuthor, blogTitle);
            if (iterator.hasNext()) {
                Node node = iterator.nextNode();
                if (!node.getProperty("blogtext").getString().equals(blogtext)) {
                    node.checkout();
                    node.setProperty("blogtext", new StringValue(blogtext));
                    session.save();
                    node.checkin();
                    System.out.println("Saved blog update");
                }
            } else {
                throw new ItemNotFoundException("There is no blog by " + blogAuthor + " with title " +
                        blogTitle + " Try adding it first.");
            }

        } catch (NamingException e) {
            throw new RuntimeException("Could not bind repository", e);
        } catch (RepositoryException e) {
            throw new RuntimeException("A RepositoryException occured", e);
        } finally {
            if (session != null) {
                session.logout();
            }
        }
    }

    /**
     * Add a new blog entry to the repository.
     *
     * @param blogAuthor the blog author's name
     * @param blogTitle  the blog entry's title
     * @param blogText   the blog entry's text
     */
    private static void addBlogEntry(String blogAuthor,
                                     String blogTitle, String blogText) {

        if (blogAuthor == null || blogTitle == null || blogText == null) {
            throw new NullPointerException();
        }

        Session session = null;
        try {
            Repository r = getRepository();

            Credentials credentials = new SimpleCredentials(USERID, PASSWORD);
            session = r.login(credentials);

            //Find if the blog entry already exists. If so, indicate that with an exception
            Workspace workspace = session.getWorkspace();
            NodeIterator results = findBlog(workspace, blogAuthor, blogTitle);
            if (results.hasNext()) {
                throw new IllegalArgumentException("A blog entry with author " + blogAuthor +
                        " and title " + blogTitle + " already exists. Try updating it.");
            }

            Node rootNode = session.getRootNode();
            Node n = rootNode.addNode("blog");
            n.addMixin("mix:versionable");
            n.setProperty("blogtitle", new StringValue(blogTitle));
            n.setProperty("blogauthor", blogAuthor);
            n.setProperty("blogdate", new DateValue(Calendar.getInstance()));
            n.setProperty("blogtext", new StringValue(blogText));
            session.save();
            n.checkin();

            System.out.println("Entered blog for : " + blogAuthor);
            System.out.println(blogTitle + ": " + blogText);

        } catch (NamingException e) {
            throw new RuntimeException("Could not bind repository", e);
        } catch (RepositoryException e) {
            throw new RuntimeException("A RepositoryException occured", e);
        } finally {
            if (session != null) {
                session.logout();
            }
        }
    }

    private static void printEntriesByBlogAuthor(String blogAuthor) {
        if (blogAuthor == null) {
            throw new NullPointerException();
        }

        Session session = null;
        try {
            Repository r = getRepository();

            Credentials credentials = new SimpleCredentials(USERID, PASSWORD);
            session = r.login(credentials);

            System.out.println("Blog entries by " + blogAuthor + ": ");

            Workspace workspace = session.getWorkspace();
            QueryManager queryMan = workspace.getQueryManager();

            //Specify an XPATH query. SQL could be used as well.
            StringBuffer qstr = new StringBuffer("//blog[@blogauthor = '");
            qstr.append(blogAuthor);
            qstr.append("']");
            Query q = queryMan.createQuery(qstr.toString(), Query.XPATH);
            QueryResult results = q.execute();
            NodeIterator it = results.getNodes();
            while (it.hasNext()) {
                Node node = it.nextNode();
                Property blogtitle = node.getProperty("blogtitle");
                Property blogtext = node.getProperty("blogtext");
                Property blogdate = node.getProperty("blogdate");

                System.out.println("Subject: " + blogtitle.getString());
                System.out.println("Posted: " + df.format(blogdate.getDate().getTime()));
                System.out.println(blogtext.getString() + "\n");
            }
        } catch (NamingException e) {
            throw new RuntimeException("Could not bind repository", e);
        } catch (RepositoryException e) {
            throw new RuntimeException("A RepositoryException occured", e);
        } finally {
            if (session != null) {
                session.logout();
            }
        }
    }

    private static void printVersions(String blogAuthor, String blogTitle) {
        if (blogAuthor == null || blogTitle == null) {
            throw new NullPointerException();
        }

        Session session = null;
        try {
            Repository r = getRepository();

            Credentials credentials = new SimpleCredentials(USERID, PASSWORD);
            session = r.login(credentials);

            Workspace workspace = session.getWorkspace();
            NodeIterator iterator = findBlog(workspace, blogAuthor, blogTitle);
            if (iterator.hasNext()) {
                Node node = iterator.nextNode();
                Property blogtitle = node.getProperty("blogtitle");
                Property blogtext = node.getProperty("blogtext");
                Property blogdate = node.getProperty("blogdate");

                System.out.println("Subject: " + blogtitle.getString());
                System.out.println("Posted: " + df.format(blogdate.getDate().getTime()));
                System.out.println("Current text: " + blogtext.getString() + "\n");

                VersionHistory versions = node.getVersionHistory();
                VersionIterator versionIterator = versions.getAllVersions();
                while (versionIterator.hasNext()) {
                    //Each version has  "frozen" child node that contains the original node for which
                    //this is a version.
                    Version version = versionIterator.nextVersion();
                    NodeIterator frozenIterator = version.getNodes("jcr:frozenNode");
                    if (frozenIterator.hasNext()) {
                        Node frosty = frozenIterator.nextNode();
                        System.out.println("Version saved on " + df.format(version.getCreated().getTime()));
                        System.out.println("   " + frosty.getProperty("blogtext").getString());
                    }
                }
            } else {
                System.out.println("There is no blog entry with author " + blogAuthor +
                        " and subject " + blogTitle + " Try adding one.");
            }

        } catch (NamingException e) {
            throw new RuntimeException("Could not bind repository", e);
        } catch (RepositoryException e) {
            throw new RuntimeException("A RepositoryException occured", e);
        } finally {
            if (session != null) {
                session.logout();
            }
        }
    }

    private static NodeIterator findBlog(Workspace workspace, String blogAuthor, String blogTitle)
            throws RepositoryException {
        if (workspace == null || blogAuthor == null || blogTitle == null) {
            throw new NullPointerException();
        }
        QueryManager queryMan = workspace.getQueryManager();

        //Specify an XPATH query. SQL could be used as well.
        StringBuffer qstr = new StringBuffer("//blog[@blogauthor = '");
        qstr.append(blogAuthor);
        qstr.append("' and @blogtitle = '");
        qstr.append(blogTitle);
        qstr.append("']");
        Query q = queryMan.createQuery(qstr.toString(), Query.XPATH);
        QueryResult results = q.execute();
        NodeIterator iterator = results.getNodes();
        return iterator;
    }

    private static Repository getRepository() throws NamingException, RepositoryException {
        if (ctx == null) {
            bindRepository();
        }
        Repository r = (Repository) ctx.lookup(REPO_NAME);
        return r;
    }


    /**
     * Bind the repository in a JNDI context so that other clients can
     * access the repository via a JNDI lookup. This code would likely
     * appear in a Web application, and not in a standalone program.
     * It is shown here to provide an example how one might look up
     * a repository instance from a naming registry.
     */
    private static void bindRepository() throws NamingException, RepositoryException {
        Hashtable environment = new Hashtable();
        environment.put(Context.INITIAL_CONTEXT_FACTORY,
                "org.apache.jackrabbit.core.jndi.provider.DummyInitialContextFactory");
        environment.put(Context.PROVIDER_URL, "localhost");
        ctx = new InitialContext(environment);
        RegistryHelper.registerRepository(ctx, REPO_NAME, CONFIG_FILE, REPO_HOME_DIR, true);
    }

    private static String usage() {
        StringBuffer b = new StringBuffer("The following command-line switches are possible:\n");
        b.append("-add BLOGGER BLOGTITLE BLOGTEXT</code> Add a blog entry\n");
        b.append("-list BLOGGER</code> List entries written by the specified blogger\n");
        b.append("-versions BLOGGER BLOGTITLE</code> List all versions of the blog entry with");

        return b.toString();

    }

}
